/*******************************************************************************
*    SMS Professional 1.2 for PalmOS                                           *
*    Copyright (C) 2006-2007 Luca Bertoncello                                  *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <PalmOS.h>
#include <TelephonyMgr.h>
#include "smsPro.h"
#include "smsLib.h"
#include "addBookLib.h"

extern DmOpenRef SmsDB;
extern LocalID SmsDBID;
extern ProgressPtr prg;
extern UInt16 stage;
UInt16 mTelRefNum;
TelAppID mTelAppId;
extern Preferences gAppPrefs;

Boolean ProgressCallback(PrgCallbackDataPtr data)
{
  MemHandle H;
  Char buf[255];

  data->bitmapId = ProgressSendBitmap;
  switch (data->stage)
  {
    case STAGEINIT:
      H = DmGetResource(strRsc, ProgressInit);
      StrCopy(buf, (char*)MemHandleLock(H));
      data->bitmapId = ProgressInitBitmap;
    break;

    case STAGESELSTORAGE:
      H = DmGetResource(strRsc, ProgressSelectStorage);
      StrPrintF(buf, (char*)MemHandleLock(H), data->message);
      data->bitmapId = ProgressInitBitmap;
    break;

    case STAGEGETSMSCOUNT:
      H = DmGetResource(strRsc, ProgressGetMessageCount);
      StrCopy(buf, (char*)MemHandleLock(H));
      data->bitmapId = ProgressInitBitmap;
    break;

    case STAGEGETDATASIZE:
      H = DmGetResource(strRsc, ProgressGetDataSize);
      StrCopy(buf, (char*)MemHandleLock(H));
      data->bitmapId = ProgressInitBitmap;
    break;

    case STAGERECEIVESMS:
      H = DmGetResource(strRsc, ProgressReceive);
      StrPrintF(buf, (char*)MemHandleLock(H), data->message);
      data->bitmapId = ProgressReceiveBitmap;
    break;

    case STAGESENDSMS:
      H = DmGetResource(strRsc, ProgressSend);
      StrPrintF(buf, (char*)MemHandleLock(H), data->message);
    break;

    default:
      H = DmGetResource(strRsc, ProgressFinish);
      StrCopy(buf, (char*)MemHandleLock(H));
    break;
  }
  MemHandleUnlock(H);
  DmReleaseResource(H);
  StrCopy(data->textP, buf);

  return true;
}

int isMessageAlreadyInDB(Char *sender, TelSmsDateTimeType time)
{
  int nRecords, i;
  MemHandle recH;
  DBRecordType *sms;

  nRecords = DmNumRecords(SmsDB);
  for(i = 0; i < nRecords; i++)
  {
    recH = DmQueryRecord(SmsDB, i);
    sms = (DBRecordTypePtr) MemHandleLock(recH);
    MemHandleUnlock(recH);
    if(sms->arriving.absolute == time.absolute && sms->arriving.dateTime == time.dateTime)
      return 1;
  }
  return 0;
}

void receiveSMS(int nMsg, UInt16 dataSize)
{
  TelSmsDeliveryMessageType msg;
  TelSmsDeleteMessageType dMsg;
  Err err;
  UInt16 index;
  int ret;
  MemHandle recH;
  MemPtr recP;
  DBRecordType dbRecord;
  Char name[255];

  MemSet(&msg, sizeof(TelSmsDeliveryMessageType), 0);
  msg.index = nMsg;
  msg.dataSize = dataSize + 1;
  msg.data = (unsigned char *) MemPtrNew(msg.dataSize);
  MemSet(msg.data, msg.dataSize, 0);
  msg.originatingAddressSize = kTelMaxPhoneNumberLen + 1;
  msg.originatingAddress = (char*) MemPtrNew(msg.originatingAddressSize);
  MemSet(msg.originatingAddress, msg.originatingAddressSize, 0);
  msg.standardType = kTelNwkGSM;
  msg.extensionsCount = 5;
  msg.extensionsP = (TelSmsExtensionType*) MemPtrNew(msg.extensionsCount * sizeof(TelSmsExtensionType));
  msg.advancedParams.advancedGSM.serviceCenterNumberSize = kTelMaxPhoneNumberLen + 1;
  msg.advancedParams.advancedGSM.serviceCenterNumber = (char*) MemPtrNew(kTelMaxPhoneNumberLen + 1);
  err = TelSmsReadMessage(mTelRefNum, mTelAppId, &msg, NULL);
/*err = 0; // Just for test...
StrCopy(msg.data, "Das ist ein Test...");
StrCopy(msg.originatingAddress, "+49174123456");
msg.timeStamp.absolute = true;
msg.timeStamp.dateTime = TimGetSeconds(); */
  if(err == 0)
  {
    if(isMessageAlreadyInDB(msg.originatingAddress, msg.timeStamp) == 0)
    {
      index = DmNumRecords(SmsDB);
      if(!(recH = DmNewRecord(SmsDB, &index, sizeof(DBRecordType))))
      {
        FrmCustomAlert(ErrorAlert, "Unable to create record", "", "");
        return;
      }

      MemSet(&dbRecord, sizeof(DBRecordType), 0);
      dbRecord.mailbox = INBOX; // received SMS are always in INBOX
      dbRecord.flags = 0;

      StrCopy(dbRecord.phone, msg.originatingAddress);
      ret = getNameFromNumber(dbRecord.phone, name);
      if(ret != 0)
        StrCopy(dbRecord.name, name);
      else
        StrCopy(dbRecord.name, "");

      TimSecondsToDateTime(msg.timeStamp.dateTime, &dbRecord.dateTime);
      dbRecord.arriving.absolute = msg.timeStamp.absolute;
      dbRecord.arriving.dateTime = msg.timeStamp.dateTime;
      StrCopy(dbRecord.text, msg.data);

      recP = MemHandleLock(recH);
      DmWrite(recP, 0, &dbRecord, sizeof(DBRecordType));
      MemPtrUnlock(recP);

      DmReleaseRecord(SmsDB, index, true);

      if(gAppPrefs.delAfterReceive)
      {
        dMsg.messageType = kTelSmsMessageAllTypes;
        dMsg.index = msg.index;
        TelSmsDeleteMessage(mTelRefNum, mTelAppId, &dMsg, NULL);
      }
    }
  }
  else
  {
    StrPrintF(name, "Unable to get SMS %d", nMsg);
    FrmCustomAlert(ErrorAlert, name, "", "");
  }
  MemPtrFree(msg.data);
  MemPtrFree(msg.originatingAddress);
  MemPtrFree(msg.extensionsP);
  MemPtrFree(msg.advancedParams.advancedGSM.serviceCenterNumber);
}

void sendReceiveSMS()
{
  UInt16 dataSize;
  MemHandle recH;
  MemPtr recP;
  DBRecordType dbRecord, *sms;
  Char buf[20];
  int nRecords, i, j, nMsg, maxMsg;
  TelSmsGetAvailableStorageType stg;
  TelSmsGetMessageCountType cnt;
  Err err;
  UInt16 nAppPrefsSize;

  nAppPrefsSize = sizeof(Preferences);
  if(PrefGetAppPreferences(appID, 0, &gAppPrefs, &nAppPrefsSize, true) == noPreferenceFound)
    MemSet(&gAppPrefs, sizeof(Preferences), 0);

// Init
  initSMSSystem();
  stage = STAGEINIT;
  if((prg = PrgStartDialog("SMS Professional", &ProgressCallback, NULL)) == NULL)
    FrmCustomAlert(ErrorAlert, "Unable to start progress dialog", "", "");

  PrgUpdateDialog(prg, 0, stage, NULL, true);
  stg.count = 10;
  stg.storagesP = (UInt8*) MemPtrNew(stg.count);
  err = TelSmsGetAvailableStorage(mTelRefNum, mTelAppId, &stg, NULL); 
  if(err != errNone)
  {
    PrgStopDialog(prg, false);
    prg = NULL;
    finishSMSSystem();
    FrmCustomAlert(ErrorAlert, "Unable to get SMS Storage", "", "");
    MemPtrFree(stg.storagesP);
    return;
  }

  if(prg != NULL)
  {
// Sending...
    stage = STAGESENDSMS;
    nRecords = DmNumRecords(SmsDB);
    for(maxMsg = i = 0; i < nRecords; i++)
    {
      recH = DmQueryRecord(SmsDB, i);
      sms = (DBRecordTypePtr) MemHandleLock(recH);
      MemHandleUnlock(recH);
      if(sms->mailbox == OUTBOX)
        maxMsg++;
    }
    for(nMsg = 1, i = 0; i < nRecords && prg != NULL; i++)
    {
      recH = DmQueryRecord(SmsDB, i);
      sms = (DBRecordTypePtr) MemHandleLock(recH);
      MemHandleUnlock(recH);

      recH = DmGetRecord(SmsDB, i);
      if(sms->mailbox == OUTBOX)
      { // Sends the SMS
        StrPrintF(buf, "%d/%d", nMsg, maxMsg);
        PrgUpdateDialog(prg, 0, stage, buf, true);
        nMsg++;

        dbRecord.mailbox = SENT;
        StrCopy(dbRecord.phone, sms->phone);
        StrCopy(dbRecord.name, sms->name);
        TimSecondsToDateTime(TimGetSeconds(), &dbRecord.dateTime);
        StrCopy(dbRecord.text, sms->text);
        dbRecord.flags = sms->flags;
        sendSMS(dbRecord);
        recP = MemHandleLock(recH);
        DmWrite(recP, 0, &dbRecord, sizeof(DBRecordType));
        MemPtrUnlock(recP);
      }
      DmReleaseRecord(SmsDB, i, true);

      if(PrgUserCancel(prg))
        prg = NULL;
    }
  }

// Receiving
  for(j = 0; j < stg.count && prg != NULL; j++)
  {
    stage = STAGESELSTORAGE;
    StrPrintF(buf, "%d/%d", (j + 1), stg.count);
    PrgUpdateDialog(prg, 0, stage, buf, true);
    err = TelSmsSelectStorage(mTelRefNum, mTelAppId, stg.storagesP[j], NULL); 
    if(err != errNone)
    {
      PrgStopDialog(prg, false);
      prg = NULL;
      finishSMSSystem();
      FrmCustomAlert(ErrorAlert, "Unable to select SMS Storage", "", "");
      MemPtrFree(stg.storagesP);
      return;
    }

    if(PrgUserCancel(prg))
      prg = NULL;

    if(prg != NULL)
    {
      stage = STAGEGETSMSCOUNT;
      PrgUpdateDialog(prg, 0, stage, NULL, true);
      cnt.messageType = kTelSmsMessageTypeDelivered;
      err = TelSmsGetMessageCount(mTelRefNum, mTelAppId, &cnt, NULL);
      if(err != errNone)
      {
        PrgStopDialog(prg, false);
        prg = NULL;
        finishSMSSystem();
        FrmCustomAlert(ErrorAlert, "Unable to get SMS Count", "", "");
        MemPtrFree(stg.storagesP);
        return;
      }

      stage = STAGEGETDATASIZE;
      PrgUpdateDialog(prg, 0, stage, NULL, true);
      TelSmsGetDataMaxSize(mTelRefNum, mTelAppId, &dataSize, NULL );

      for(i = 0; i < cnt.count && prg != NULL; i++)
      {
        stage = STAGERECEIVESMS;
        StrPrintF(buf, "%d/%d", (i + 1), cnt.count);
        PrgUpdateDialog(prg, 0, stage, buf, true);
        receiveSMS(i, dataSize);

        if(PrgUserCancel(prg))
          prg = NULL;
      }
    }
  }

  stage = STAGEFINISH;
  PrgUpdateDialog(prg, 0, stage, NULL, true);

  MemPtrFree(stg.storagesP);
  PrgStopDialog(prg, false);
  prg = NULL;
  finishSMSSystem();
}

void initSMSSystem()
{
  Err  err;
  MemHandle H;

  err = SysLibFind(kTelMgrLibName, &mTelRefNum);
  if(err !=0)
  {
    err = SysLibLoad(kTelMgrDatabaseType, kTelMgrDatabaseCreator, &mTelRefNum);
    if(err)
    {
      H = DmGetResource(strRsc, UnableToLoadTelLib);
      FrmCustomAlert(ErrorAlert, (char*)MemHandleLock(H), "", "");
      return;
    }
  }

  err = TelOpen(mTelRefNum, kTelMgrVersion, &mTelAppId);
  if(err)
  {
      H = DmGetResource(strRsc, UnableToOpenTel);
      FrmCustomAlert(ErrorAlert, (char*)MemHandleLock(H), "", "");
    return;
  }

  if(TelIsSmsServiceAvailable(mTelRefNum, mTelAppId, NULL) != errNone)
  {
    H = DmGetResource(strRsc, UnableToGetSMSService);
    FrmCustomAlert(ErrorAlert, (char*)MemHandleLock(H), "", "");
    return;
  }
}

void finishSMSSystem()
{
  if(TelClose(mTelRefNum, mTelAppId) != telErrLibStillInUse)
    SysLibRemove(mTelRefNum);
}

void sendSMS(DBRecordType dbRecord)
{
  Err  err;
  TelSmsSendMessageType smsMessage;
  TelSmsExtensionType smsExtension;

  smsMessage.message.networkDeliveryRequest = (dbRecord.flags & SENDCONFIRMATIONFLAG) ? true : false;

  smsMessage.message.version = kTelSmsAPIVersion;
  smsMessage.message.destinationAddress = dbRecord.phone;
  smsMessage.message.destinationAddressSize = StrLen(dbRecord.phone);

  smsMessage.message.dataSize = StrLen(dbRecord.text);
  smsMessage.message.data = (unsigned char *) dbRecord.text;
  smsMessage.message.dataCodingScheme =kTelSmsDefaultGSMEncoding;

  smsMessage.message.validityPeriod.absolute = false;
  smsMessage.message.validityPeriod.dateTime = (UInt32) 86400;

  smsMessage.message.standardType = kTelNwkGSM;
  smsMessage.message.advancedParams.advancedGSM.protocolId = kTelSmsDefaultProtocol;
  smsMessage.message.advancedParams.advancedGSM.rejectDuplicatedRequest = true;
  smsMessage.message.advancedParams.advancedGSM.replyPath = false;
  smsMessage.message.advancedParams.advancedGSM.serviceCenterNumber = NULL;
  smsMessage.message.advancedParams.advancedGSM.serviceCenterNumberSize = 0;

  smsMessage.message.extensionsCount = 1;
  smsMessage.message.extensionsP = &smsExtension;

  smsExtension.extensionTypeId = kTelSmsMultiPartExtensionTypeId;
  smsExtension.extension.mp.bytesSent = 0;
  smsExtension.extension.mp.partCurrent = 0;
  smsExtension.extension.mp.partCount = 0;
  smsExtension.extension.mp.partId = 0;

  err = TelSmsSendMessage( mTelRefNum, mTelAppId, &smsMessage, NULL);

  while(!err && smsMessage.message.extensionsP[0].extension.mp.bytesSent != smsMessage.message.dataSize)
  {
    err = TelSmsSendMessage( mTelRefNum, mTelAppId, &smsMessage, NULL);
  }
}
