/*******************************************************************************
*    SMS Professional 1.2 for PalmOS                                           *
*    Copyright (C) 2006-2007 Luca Bertoncello                                  *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <PalmOS.h>
#include <ConnectionMgr.h>
#include <SystemMgr.h>
#include <List.h>
#include <PalmTypes.h>
#include <PalmCompatibility.h>
#include <System/SystemPublic.h>
#include <UI/UIPublic.h>
#include <TelephonyMgr.h>
#include <unix_stdlib.h>
#include "smsPro.h"
#include "smsLib.h"
#include "addBookLib.h"

static FormPtr gpForm;
int currentMailbox = INBOX;
UInt16 currentMessage = -1;
DmOpenRef SmsDB = NULL;
LocalID SmsDBID;
UInt16 dbCard;
static Char addrLookup[1024], selectedPhone[255], selectedName[255], curPhone[255], curName[255];
Char **numbersList, newSelectedNumber[255];
int nNumbers;
ProgressPtr prg = NULL;
UInt16 stage;
static Char **recordIds = NULL;
static int nElements = 0;
Preferences gAppPrefs;

void CallPanel(UInt32 appCreator) 
{ 
  LocalID dbID;
  DmSearchStateType searchState;
  Err err;
  UInt16 cno;

  DmGetNextDatabaseByTypeCreator(true, &searchState, sysFileTPanel, appCreator, true, &cno, &dbID);
  ErrNonFatalDisplayIf(!dbID, "Could not find app");
  if(dbID)
  {
    err = SysUIAppSwitch(cno, dbID, sysAppLaunchCmdPanelCalledFromApp, NULL);
    ErrNonFatalDisplayIf(err, "Could not launch app");
  }
}

static Err StartApplication() 
{
  Err retcode = 0;
  UInt16 attributes;
  UInt32 locks;

  SmsDB = DmOpenDatabaseByTypeCreator(DBType, appID, dmModeReadWrite | dmModeShowSecret);
  if(SmsDB)
    DmOpenDatabaseInfo(SmsDB, &SmsDBID, NULL, NULL, &dbCard, NULL);
  else
  {
    if((retcode = DmCreateDatabase(0, DBName, appID, DBType, false)))
      return retcode;

    if (!(SmsDB = DmOpenDatabaseByTypeCreator(DBType, appID, dmModeReadWrite | dmModeShowSecret)))
      return dmErrCantOpen;

    DmOpenDatabaseInfo(SmsDB, &SmsDBID, NULL, NULL, &dbCard, NULL);
    DmDatabaseInfo(dbCard, SmsDBID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
    attributes |= dmHdrAttrBackup;
    DmSetDatabaseInfo(dbCard, SmsDBID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
  }
  DmGetDatabaseLockState(SmsDB, NULL, NULL, &locks);
  DmDatabaseProtect(dbCard, SmsDBID, (locks > 0) ? false : true);

  FrmGotoForm(MainForm);
  return 0;
}

static void StopApplication() 
{
  UInt32 locks;
  Boolean protect = false;
  char buf[40];
  int i;

  FrmCloseAllForms();

  if(recordIds != NULL)
  {
    for (i = 0; i < nElements; i++)
      MemPtrFree(recordIds[i]);
    MemPtrFree(recordIds);
  }

  DmGetDatabaseLockState(SmsDB, NULL, NULL, &locks);
  if((protect = (locks > 0)))
  {
    StrPrintF(buf, "%ld record(s) still locked.", locks);
    FrmCustomAlert(ErrorAlert, buf, "Database will be closed protected.", "If you don't unlock every record, you WON'T be allowed to delete the database.");
  }
  DmDatabaseProtect(dbCard, SmsDBID, protect);

  DmCloseDatabase(SmsDB);
}

static void drawList(UInt16 row, RectangleType *bounds, char **itemsText, struct ListType *listP)
{  
  Char date[30], year[6];
  MemHandle memH;
  DBRecordTypePtr sms;
  UInt16 recId;

  recId = atoi(itemsText[row]);
  memH = DmQueryRecord(SmsDB, recId);
  sms = (DBRecordTypePtr) MemHandleLock(memH);
  StrPrintF(year, "%d", sms->dateTime.year);
  if(StrLen(year) > 2)
  {
    year[0] = year[2];
    year[1] = year[3];
    year[2] = 0;
  }
  StrPrintF(date, "%02d.%02d.%s %02d:%02d", sms->dateTime.day, sms->dateTime.month, year, sms->dateTime.hour, sms->dateTime.minute);
  WinDrawChars(date, StrLen(date), bounds->topLeft.x, bounds->topLeft.y);
  if(StrCompare(sms->name, "") != 0)
    WinDrawChars(sms->name, StrLen(sms->name), bounds->topLeft.x + 62, bounds->topLeft.y);
  else
    WinDrawChars(sms->phone, StrLen(sms->phone), bounds->topLeft.x + 62, bounds->topLeft.y);
  WinDrawLine(62, bounds->topLeft.y, 62, bounds->topLeft.y + bounds->extent.y);
  MemHandleUnlock(memH);
}

Char* GetField(UInt16 formID, UInt16 fieldID)
{
  FormPtr frm;
  FieldPtr fld;
  Char *p;

  frm = FrmGetFormPtr(formID);
  fld = (FieldPtr)FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
  p = FldGetTextPtr(fld);

  return (p ? p : "");
}

void SetField(UInt16 formID, UInt16 fieldID, MemPtr str)
{
  FormPtr frm;
  FieldPtr fld;
  CharPtr p;
  VoidHand h;

  frm = FrmGetFormPtr(formID);
  fld = (FieldPtr) FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
  h = (VoidHand) FldGetTextHandle(fld);
  if(h == NULL)
  {
    h = MemHandleNew(FldGetMaxChars(fld)+1);
    ErrFatalDisplayIf(!h, "No Memory");
  }

  p = (CharPtr) MemHandleLock(h);
  StrCopy(p, str);
  MemHandleUnlock(h);
  FldSetTextHandle(fld, (Handle) h);
}

void drawNewSmsForm(Char *toPhone, Char *toName)
{
  FormPtr frm = FrmGetFormPtr(SmsForm);
  FieldAttrType attrTel, attrText;
  Char buf[255];
  UInt16 nAppPrefsSize;

  nAppPrefsSize = sizeof(Preferences);
  if(PrefGetAppPreferences(appID, 0, &gAppPrefs, &nAppPrefsSize, true) == noPreferenceFound)
    MemSet(&gAppPrefs, sizeof(Preferences), 0);
  CtlSetValue(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, sendConfirmation)), gAppPrefs.sendConf);

  FrmHideObject(frm, FrmGetObjectIndex(frm, idLabelFrom));
  FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonSave));
  FrmHideObject(frm, FrmGetObjectIndex(frm, idLabelTo));
  FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonAnswer));
  FrmShowObject(frm, FrmGetObjectIndex(frm, idButtonSend));
  FrmShowObject(frm, FrmGetObjectIndex(frm, idButtonSearch));
  FrmSetMenu(frm, SmsMenu);
  FldGetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)), &attrTel);
  FldGetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText)), &attrText);
  attrText.singleLine = 0;
  attrText.editable = 1;
  attrText.autoShift = true;
  attrTel.editable = 1;
  FldSetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText)), &attrText);
  FldSetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)), &attrTel);
  if(StrCompare(toPhone, "") == 0)
    StrCopy(buf, "");
  else
  {
    StrCopy(selectedPhone, toPhone);
    StrCopy(selectedName, toName);
    if(StrCompare(toName, "") != 0)
      StrPrintF(buf, "%s (%s)", toName, toPhone);
    else
      StrPrintF(buf, "%s", toPhone);
  }
  SetField(SmsForm, idFieldTelNumber, buf);
  SetField(SmsForm, idFieldText, "");
  FldDrawField(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)));
  FldDrawField(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText)));
  SclSetScrollBar(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idTextScrollbar)), 0, 0, 0, 0);
  FldSetScrollPosition(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText)), 0);
}

void drawSmsForm()
{
  FormPtr frm = FrmGetFormPtr(SmsForm);
  MemHandle memH;
  DBRecordTypePtr sms;
  FieldAttrType attrTel, attrText;
  FieldPtr fld = (FieldPtr) FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText));
  Char buf[255];
  Word scrollPos, textHeight, fieldHeight;

  memH = DmQueryRecord(SmsDB, currentMessage);
  sms = (DBRecordTypePtr) MemHandleLock(memH);
  FldGetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)), &attrTel);
  FldGetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)), &attrText);
  attrTel.editable = 0;
  attrText.singleLine = 0;
  attrText.editable = 0;
  attrText.autoShift = 1;

  FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonSend));
  FrmShowObject(frm, FrmGetObjectIndex(frm, idButtonSave));
  if(StrCompare(sms->name, "") != 0)
    StrPrintF(buf, "%s (%s)", sms->name, sms->phone);
  else
    StrPrintF(buf, "%s", sms->phone);
  StrCopy(curName, sms->name);
  StrCopy(curPhone, sms->phone);
  StrCopy(selectedPhone, sms->phone);
  StrCopy(selectedName, sms->name);
  SetField(SmsForm, idFieldTelNumber, buf);
  SetField(SmsForm, idFieldText, sms->text);
  CtlSetValue(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, sendConfirmation)), (sms->flags & SENDCONFIRMATIONFLAG) ? 1 : 0);
  currentMailbox = sms->mailbox;
  switch(currentMailbox)
  {
    case INBOX:
    {
      FrmHideObject(frm, FrmGetObjectIndex(frm, idLabelTo));
      FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonSearch));
      FrmShowObject(frm, FrmGetObjectIndex(frm, idLabelFrom));
      FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonSave));
      FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonSend));
      FrmShowObject(frm, FrmGetObjectIndex(frm, idButtonAnswer));
      FldRecalculateField(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText)), true);
      FldSetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)), &attrTel);
      FldSetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText)), &attrText);
      CtlSetEnabled(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, sendConfirmation)), false);
      FrmSetMenu(frm, SmsMenuRO);
    }
    break;
    case OUTBOX:
    {
      FrmHideObject(frm, FrmGetObjectIndex(frm, idLabelFrom));
      FrmHideObject(frm, FrmGetObjectIndex(frm, idLabelTo));
      FrmShowObject(frm, FrmGetObjectIndex(frm, idButtonSearch));
      FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonSend));
      FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonAnswer));
      FrmShowObject(frm, FrmGetObjectIndex(frm, idButtonSave));
      CtlSetEnabled(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, sendConfirmation)), true);
      FrmSetMenu(frm, SmsMenu);
    }
    break;
    case SENT:
    {
      FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonSearch));
      FrmHideObject(frm, FrmGetObjectIndex(frm, idLabelFrom));
      FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonAnswer));
      FrmHideObject(frm, FrmGetObjectIndex(frm, idButtonSend));
      FrmShowObject(frm, FrmGetObjectIndex(frm, idLabelTo));
      CtlSetEnabled(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idButtonSave)), false);
      CtlSetEnabled(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, sendConfirmation)), false);
      FldSetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)), &attrTel);
      FldSetAttributes(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText)), &attrText);
      FrmSetMenu(frm, SmsMenuRO);
    }
    break;
  }

  FldDrawField(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)));
  FldDrawField(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText)));
  FldGetScrollValues(fld, &scrollPos, &textHeight, &fieldHeight);
  if(textHeight > fieldHeight)
    SclSetScrollBar(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idTextScrollbar)), scrollPos, 0, textHeight - fieldHeight, fieldHeight);
  else
    SclSetScrollBar(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idTextScrollbar)), scrollPos, 0, 0, 0);
  FldSetScrollPosition(fld, 0);
  MemHandleUnlock(memH);
}

void drawForm(int mailBox)
{
  MemHandle memH;
  UInt16 nRecords, i;
  DBRecordTypePtr sms;
  FormPtr frm = FrmGetFormPtr(MainForm);
  ListPtr list = (ListPtr)FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, Messages));
  Char buf[10];

  currentMailbox = mailBox;
  nRecords = DmNumRecords(SmsDB);
  if(recordIds != NULL)
  {
    for (i = 0; i < nElements; i++)
      MemPtrFree(recordIds[i]);
    MemPtrFree(recordIds);
  }
  nElements = 0;
  recordIds = (Char **) MemPtrNew(nRecords * sizeof(Char *));
  for (i = 0; i < nRecords; i++)
  {
    memH = DmQueryRecord(SmsDB, i);
    sms = (DBRecordTypePtr) MemHandleLock(memH);
    if(sms->mailbox == currentMailbox)
    {
      StrPrintF(buf, "%d", i);
      recordIds[nElements] = (char *) MemPtrNew(StrLen(buf) + 1);
      StrCopy(recordIds[nElements], buf);
      nElements++;
    }
    MemHandleUnlock(memH);
  }
  LstSetListChoices(list, recordIds, nElements);
  LstSetDrawFunction(list, (ListDrawDataFuncPtr) drawList);
  LstSetSelection(list, noListSelection);
  LstDrawList(list);

  WinDrawLine(62, 50, 62, 138);
  CtlSetEnabled(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idButtonOpen)), false);
  CtlSetEnabled(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idButtonDelete)), false);
}

void deleteSMS()
{
  FormPtr frm = FrmGetFormPtr(MainForm);
  Int16 sel;
  ListType *lst;
  UInt16 recId;

  lst = (ListType*) FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, Messages));
  sel = LstGetSelection(lst);
  if(sel != noListSelection)
  {
    recId = atoi(LstGetSelectionText(lst, sel));
    DmRemoveRecord(SmsDB, recId);
  }
}

void saveSMS()
{
  UInt16 index;
  MemHandle recH;
  MemPtr recP;
  DBRecordType dbRecord;
  Char n[255], t[255];
  int l, k;
  UInt32 flags = 0;
  FormPtr frm = FrmGetFormPtr(SmsForm);

  if(CtlGetValue(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, sendConfirmation))))
    flags |= SENDCONFIRMATIONFLAG;

  if(currentMessage == -1) // new SMS
  {
    index = DmNumRecords(SmsDB);
    if(!(recH = DmNewRecord(SmsDB, &index, sizeof(DBRecordType))))
    {
      FrmCustomAlert(ErrorAlert, "Unable to create record", "", "");
      return;
    }
  }
  else
  {
    if(!(recH = (MemHandle) DmGetRecord(SmsDB, currentMessage)))
    {
      FrmCustomAlert(ErrorAlert, "Unable to update record", "", "");
      return;
    }
  }

  dbRecord.mailbox = OUTBOX;
  TimSecondsToDateTime(TimGetSeconds(), &dbRecord.dateTime);
  if(StrCompare(selectedPhone, "") == 0)
  {
    StrCopy(t, GetField(SmsForm, idFieldTelNumber));
    MemSet(n, sizeof(Char) * 255, 0);
    for(k = l = 0; l < StrLen(t); l++)
      if(t[l] == '+' || (t[l] >= '0' && t[l] <= '9'))
        n[k++] = t[l];
    StrCopy(dbRecord.phone, n);
    StrCopy(dbRecord.name, "");
  }
  else
  {
    StrCopy(dbRecord.phone, selectedPhone);
    StrCopy(dbRecord.name, selectedName);
  }
  StrCopy(dbRecord.text, GetField(SmsForm, idFieldText));
  dbRecord.flags = flags;

  recP = MemHandleLock(recH);
  DmWrite(recP, 0, &dbRecord, sizeof(DBRecordType));
  MemPtrUnlock(recP);

  DmReleaseRecord(SmsDB, index, true);
}

void openSMS(int displayNew)
{
  FormPtr frm = FrmGetFormPtr(MainForm);
  Int16 sel;
  ListType *lst;

  if(displayNew == 0)  // Display
  {
    lst = (ListType*) FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, Messages));
    sel = LstGetSelection(lst);
    if(sel != noListSelection)
    {
      currentMessage = atoi(LstGetSelectionText(lst, sel));
      FrmPopupForm(SmsForm);
    }
  }
  else  // New SMS
  {
    currentMessage = -1;
    FrmPopupForm(SmsForm);
  }
}

void getNumber(FormPtr frm)
{
  AddrLookupParamsType params;
  MemHandle H;
  Char old[255];

  MemSet(&params, sizeof(AddrLookupParamsType), 0);
  H = DmGetResource(strRsc, AddrSearchTitle);
  params.title = ((char*)MemHandleLock(H));
  MemHandleUnlock(H);
  DmReleaseResource(H);
  H = DmGetResource(strRsc, AddrSearchButtonText);
  params.pasteButtonText = ((char*)MemHandleLock(H));
  MemHandleUnlock(H);
  DmReleaseResource(H);
  params.field1 = addrLookupSortField;
  params.field2 = addrLookupNoField;
  params.field2Optional = true;
  params.userShouldInteract = true;
  params.formatStringP = "^name ^first|^work|^home|^fax|^other|^mobile|^main";
  StrCopy(old, GetField(SmsForm, idFieldTelNumber));
  SetField(SmsForm, idFieldTelNumber, "");
  PhoneNumberLookupCustom(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)), &params, false);

  if(params.uniqueID != 0)
  {
    StrCopy(addrLookup, GetField(SmsForm, idFieldTelNumber));
    SetField(SmsForm, idFieldTelNumber, "");
    FldDrawField(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)));
    FrmPopupForm(SelectNumberForm);
  }
  else
    SetField(SmsForm, idFieldTelNumber, old);
}

int isPhoneNumber(Char *str)
{
  int l, ret = 1;

  for(l = 0; l < StrLen(str); l++)
    if(str[l] != '+' && str[l] != ' ' && str[l] != '(' && str[l] != ')' && str[l] != '-' && str[l] != '/' && (str[l] < '0' || str[l] > '9'))
      ret = 0;

  return ret;
}

Boolean PreferencesFormEventHandler(EventPtr event) 
{
  Boolean handled = false;
  FormPtr frm = FrmGetFormPtr(PreferencesForm);
  UInt16 nAppPrefsSize;

  switch(event->eType)
  {
    case frmOpenEvent:
      nAppPrefsSize = sizeof(Preferences);
      if(PrefGetAppPreferences(appID, 0, &gAppPrefs, &nAppPrefsSize, true) == noPreferenceFound)
        MemSet(&gAppPrefs, sizeof(Preferences), 0);
      CtlSetValue(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, deleteAfterReceive)), gAppPrefs.delAfterReceive);
      CtlSetValue(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, sendConfirmation)), gAppPrefs.sendConf);
      FrmDrawForm(frm);
      handled = true;
    break;

    case frmCloseEvent:
      FrmDeleteForm(frm);
      handled = true;
    break;

    case ctlSelectEvent:
      switch (event->data.ctlSelect.controlID)
      {
        case idButtonSave:
          if(CtlGetValue(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, deleteAfterReceive))))
            gAppPrefs.delAfterReceive = 1;
          else
            gAppPrefs.delAfterReceive = 0;
          if(CtlGetValue(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, sendConfirmation))))
            gAppPrefs.sendConf = 1;
          else
            gAppPrefs.sendConf = 0;
          PrefSetAppPreferences(appID, 0, versionID, &gAppPrefs, sizeof(Preferences), true);
          FrmReturnToForm(MainForm);
          handled = true;
        break;

        case idButtonCancel:
          FrmReturnToForm(MainForm);
          handled = true;
        break;
      }
    break;

    default:
    break;
  }

  return handled;
}


Boolean SelNumberFormEventHandler(EventPtr event) 
{
  Boolean handled = false;
  FormPtr frm = FrmGetFormPtr(SelectNumberForm);
  ListPtr nums;
  Char fields[7][255], t[255], n[255];
  int l, k, f;

  switch(event->eType)
  {
    case frmOpenEvent:
      for(l = 0; l < 7; l++)
        MemSet(fields[l], sizeof(Char) * 255, 0);
      for(f = k = l = 0; l < StrLen(addrLookup); l++)
      {
        if(addrLookup[l] == '|')
        {
          f++;
          k = 0;
        }
        else
          fields[f][k++] = addrLookup[l];
      }
      CtlSetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idLabelName)), fields[0]);
      numbersList = (Char **) MemPtrNew(f * sizeof(Char *));
      for(nNumbers = 0, l = 1; l < f; l++)
        if(StrLen(fields[l]) != 0 && isPhoneNumber(fields[l]))
        {
          numbersList[nNumbers] = (char *) MemPtrNew(StrLen(fields[l]) + 1);
          StrCopy(numbersList[nNumbers++], fields[l]);
        }
      LstSetListChoices(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, Numbers)), numbersList, nNumbers);
      FrmDrawForm(FrmGetFormPtr(SelectNumberForm));
      handled = true;
    break;

    case frmCloseEvent:
      for(l = 0; l < nNumbers; l++)
        MemPtrFree(numbersList[l]);
      MemPtrFree(numbersList);
      LstEraseList(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, Numbers)));
      FrmEraseForm(FrmGetFormPtr(SelectNumberForm));
      FrmDeleteForm(FrmGetFormPtr(SelectNumberForm));
      handled = true;
    break;

    case ctlSelectEvent:
      switch (event->data.ctlSelect.controlID)
      {
        case idButtonChoose:
          nums = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, Numbers));
          StrCopy(t, LstGetSelectionText(nums, LstGetSelection(nums)));
          MemSet(n, sizeof(Char) * 255, 0);
          for(k = l = 0; l < StrLen(t); l++)
            if(t[l] == '+' || (t[l] >= '0' && t[l] <= '9'))
              n[k++] = t[l];
          StrPrintF(newSelectedNumber, "%s (%s)", CtlGetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idLabelName))), n);
          StrCopy(selectedPhone, n);
          StrCopy(selectedName, CtlGetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idLabelName))));
          for(l = 0; l < nNumbers; l++)
            MemPtrFree(numbersList[l]);
          MemPtrFree(numbersList);
          FrmReturnToForm(SmsForm);
          handled = true;
        break;
        case idButtonCancel:
          for(l = 0; l < nNumbers; l++)
            MemPtrFree(numbersList[l]);
          MemPtrFree(numbersList);
          FrmReturnToForm(SmsForm);
          handled = true;
        break;
      }
    break;

    default:
    break;
  }

  return handled;
}

FieldPtr GetFocusObjectPtr(FormPtr frm)
{
  UInt16 focus;
  FormObjectKind objType;

  focus = FrmGetFocus(frm);
  if (focus == noFocus)
    return(NULL);

  objType = FrmGetObjectType(frm, focus);

  if (objType == frmFieldObj)
    return(FrmGetObjectPtr(frm, focus));
  else if (objType == frmTableObj)
    return(TblGetCurrentField(FrmGetObjectPtr(frm, focus)));

  return (NULL);
}

Boolean SmsFormEventHandler(EventPtr event) 
{
  Boolean handled = false;
  Boolean mustRetFalse = false;
  FormPtr frm = FrmGetFormPtr(SmsForm);
  FormPtr frmM = FrmGetFormPtr(MainForm);
  ListPtr list;
  ControlPtr pop;
  FieldPtr fld = (FieldPtr) FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldText));
  MemHandle H;
  Word scrollPos, textHeight, fieldHeight;

  if(StrLen(newSelectedNumber) != 0)
  {
    SetField(SmsForm, idFieldTelNumber, newSelectedNumber);
    FldDrawField(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idFieldTelNumber)));
    StrCopy(newSelectedNumber, "");
  }
  switch(event->eType)
  {
    case frmOpenEvent:
      StrCopy(selectedPhone, "");
      StrCopy(selectedName, "");
      FrmDrawForm(FrmGetFormPtr(SmsForm));
      if(currentMessage != -1)
        drawSmsForm();
      else
        drawNewSmsForm("", "");
      handled = true;
    break;

    case frmCloseEvent:
      FrmEraseForm(FrmGetFormPtr(SmsForm));
      FrmDeleteForm(FrmGetFormPtr(SmsForm));
      handled = true;
    break;

    case fldChangedEvent:
    case keyDownEvent:
    {
      FldGetScrollValues(fld, &scrollPos, &textHeight, &fieldHeight);
      switch(event->data.keyDown.chr)
      {
        case vchrPageDown:
          if(scrollPos < textHeight)
          {
            scrollPos += 6;
            FldScrollField(fld, 6, winDown);
          }
        break;

        case vchrPageUp:
          if(scrollPos > 0)
          {
            scrollPos -= 6;
            FldScrollField(fld, 6, winUp);
          }
        break;

        default:
          mustRetFalse = true;
        break;
      }
      if(event->eType == keyDownEvent && event->data.keyDown.chr == chrLineFeed)
      {
        scrollPos++;
        textHeight++;
      }
      if(textHeight > fieldHeight)
        SclSetScrollBar(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idTextScrollbar)), scrollPos, 0, textHeight - fieldHeight, fieldHeight);
      else
      {
        SclSetScrollBar(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idTextScrollbar)), 0, 0, 0, 0);
        FldSetScrollPosition(fld, 0);
      }
      FldDrawField(fld);
      if(!mustRetFalse)
        handled = true;
    }
    break;

    case sclRepeatEvent:
    {
      FldGetScrollValues(fld, &scrollPos, &textHeight, &fieldHeight);
      if (scrollPos < event->data.sclExit.newValue)
        FldScrollField(fld, event->data.sclExit.newValue - scrollPos, winDown);
      else
        FldScrollField(fld, scrollPos - event->data.sclExit.newValue, winUp);
      FldDrawField(fld);
      handled = true;
      break;
    }

    case ctlSelectEvent:
      switch (event->data.ctlSelect.controlID)
      {
        case idButtonSave:
        case idButtonSend:
          if(StrCompare(GetField(SmsForm, idFieldTelNumber), "") == 0 || StrCompare(GetField(SmsForm, idFieldText), "") == 0)
          {
            H = DmGetResource(strRsc, ErrorNoPhoneText);
            FrmCustomAlert(ErrorAlert, ((char*)MemHandleLock(H)), "", "");
            MemHandleUnlock(H);
            DmReleaseResource(H);
          }
          else
          {
            saveSMS();
            FrmReturnToForm(MainForm);
            drawForm(OUTBOX);
            list = (ListPtr)FrmGetObjectPtr(frmM, FrmGetObjectIndex(frmM, FolderList));
            pop = (ControlPtr) FrmGetObjectPtr(frmM, FrmGetObjectIndex(frmM, Folders));
            LstSetSelection(list, OUTBOX);
            CtlSetLabel(pop, LstGetSelectionText((ListPtr) FrmGetObjectPtr(frmM, FrmGetObjectIndex(frmM, FolderList)), OUTBOX));
            currentMessage = -1;
          }
          handled = true;
        break;
        case idButtonAnswer:
          currentMessage = -1;
          drawNewSmsForm(curPhone, curName);
          handled = true;
        break;
        case idButtonCancel:
          FrmReturnToForm(MainForm);
          currentMessage = -1;
          handled = true;
        break;
        case idButtonSearch:
          getNumber(frm);
          handled = true;
        break;
      }
    break;

    default:
    break;
  }

  return handled;
}

Boolean MainFormEventHandler(EventPtr event) 
{
  Boolean handled = false;
  FormPtr frm = FrmGetFormPtr(MainForm);
  ControlPtr pop;

  switch(event->eType)
  {
    case frmOpenEvent:
      gpForm = FrmGetActiveForm();
      FrmDrawForm(gpForm);
      drawForm(INBOX);
      handled = true;
    break;

    case popSelectEvent:
      pop = (ControlPtr) FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, Folders));
      CtlSetLabel(pop, LstGetSelectionText((ListPtr) FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, FolderList)), event->data.popSelect.selection));
      drawForm(event->data.popSelect.selection);
      handled = true;
    break;

    case lstSelectEvent:
      if(event->data.lstSelect.listID == Messages)
      {
        CtlSetEnabled(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idButtonOpen)), true);
        CtlSetEnabled(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, idButtonDelete)), true);
      }
      handled = true;
    break;

    case frmCloseEvent:
      FrmEraseForm(gpForm);
      FrmDeleteForm(gpForm);
      gpForm = 0;
      handled = true;
    break;

    default:
    break;
  }

  return handled;
}

static Boolean ApplicationEventHandler(EventPtr event) 
{
  Boolean handled = false;
  FormType* frmP;
  FieldPtr fld;
  ListPtr list;
  ControlPtr pop;
  int sel;

  switch(event->eType)
  {
    case nilEvent:
    {
      if(prg)
      {
        if(PrgUserCancel(prg))
        {
          PrgStopDialog(prg, false);
          prg = NULL;
        }
        else
          PrgUpdateDialog(prg, 0, stage, NULL, true);

        handled = true;
      }
    }
    break;

    case frmLoadEvent:
    {
      FormPtr pForm = FrmInitForm(event->data.frmLoad.formID);
      FrmSetActiveForm(pForm);
      switch(event->data.frmLoad.formID)
      {
        case MainForm:
          FrmSetEventHandler(pForm, MainFormEventHandler);
        break;

        case SmsForm:
          FrmSetEventHandler(pForm, SmsFormEventHandler);
        break;

        case SelectNumberForm:
          FrmSetEventHandler(pForm, SelNumberFormEventHandler);
        break;

        case PreferencesForm:
          FrmSetEventHandler(pForm, PreferencesFormEventHandler);
        break;

        default:
        break;
      }
      handled = true;
    }
    break;

    case menuEvent:
      MenuEraseStatus(NULL);
      switch(event->data.menu.itemID)
      {
      	case HelpMenuAbout:
          FrmAlert(AboutAlert);
        break;
      	case OptionMenuPreferences:
          FrmPopupForm(PreferencesForm);
        break;
      	case OptionMenuPhoneSettings:
          CallPanel(sysFileCPhonePanel);
        break;
        case SmsMenuDeleteAll:
          if((fld = GetFocusObjectPtr(FrmGetFormPtr(SmsForm))))
            FldDelete(fld, 0, FldGetTextLength(fld));
          handled = true;
        break;
        case SmsMenuUndo:
        case sysEditMenuUndoCmd:
          if((fld = GetFocusObjectPtr(FrmGetFormPtr(SmsForm))))
            FldUndo(fld);
          handled = true;
        break;
        case SmsMenuCut:
        case sysEditMenuCutCmd:
          if((fld = GetFocusObjectPtr(FrmGetFormPtr(SmsForm))))
            FldCut(fld);
          handled = true;
        break;
        case SmsMenuCopy:
        case sysEditMenuCopyCmd:
          if((fld = GetFocusObjectPtr(FrmGetFormPtr(SmsForm))))
            FldCopy(fld);
          handled = true;
        break;
        case SmsMenuPaste:
        case sysEditMenuPasteCmd:
          if((fld = GetFocusObjectPtr(FrmGetFormPtr(SmsForm))))
            FldPaste(fld);
          handled = true;
        break;
        case SmsMenuSelectAll:
        case sysEditMenuSelectAllCmd:
          if((fld = GetFocusObjectPtr(FrmGetFormPtr(SmsForm))))
            FldSetSelection(fld, 0, FldGetTextLength(fld));
          handled = true;
        break;
        case SmsMenuHelpGraffiti:
          SysGraffitiReferenceDialog(referenceDefault);
          handled = true;
        break;
      }
      handled = true;
    break;

    case ctlSelectEvent:
    {
      frmP = FrmGetActiveForm();
      switch(event->data.ctlSelect.controlID)
      {
      	case idButtonSendReceive:
      	{
          sendReceiveSMS();
          drawForm(INBOX);
          list = (ListPtr)FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, FolderList));
          pop = (ControlPtr) FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, Folders));
          LstSetSelection(list, INBOX);
          CtlSetLabel(pop, LstGetSelectionText((ListPtr) FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, FolderList)), INBOX));
      	  handled = true;
      	  break;
        }
      	case idButtonNewSMS:
      	{
          openSMS(1);
      	  handled = true;
      	  break;
        }
      	case idButtonOpen:
      	{
          openSMS(0);
      	  handled = true;
      	  break;
        }
      	case idButtonDelete:
      	{
          deleteSMS();
          frmP = FrmGetFormPtr(MainForm);
          sel = LstGetSelection(FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, FolderList)));
          drawForm(sel);
      	  handled = true;
      	  break;
        }
      }
    }
    break;

    default:
    break;
  }

  return handled;
}

static void EventLoop() 
{
  EventType event;
  UInt16 error;

  StrCopy(newSelectedNumber, "");
  do
  {
    EvtGetEvent(&event, evtWaitForever);

    if (!SysHandleEvent(&event))
    {
      if (!MenuHandleEvent(0, &event, &error))
      {
        if (!ApplicationEventHandler(&event))
          FrmDispatchEvent(&event);
      }
    }

    if(prg)
      if(PrgHandleEvent(prg, &event))
        continue;
  } while(event.eType != appStopEvent);
}

static void LookFor(FindParamsPtr params)
{
  UInt16 nRec = params->recordNum, fld = -1, length;
  UInt32 pos;
  MemHandle memH;
  DBRecordTypePtr sms;
  Boolean found;
  RectangleType rect;
  Char date[30], year[6];
  DmOpenRef mSmsDB;
  LocalID mSmsDBID;
  UInt16 mdbCard;

  if(FindDrawHeader(params, "SmsPro"))
    return;

  mSmsDB = DmOpenDatabaseByTypeCreator(DBType, appID, params->dbAccesMode);
  if(mSmsDB)
  {
    DmOpenDatabaseInfo(mSmsDB, &mSmsDBID, NULL, NULL, &mdbCard, NULL);

    while(true)
    {
      if(((nRec % 10) == 0) && EvtSysEventAvail(true))
      {
        params->more = true;
        break;
      }

      memH = DmQueryRecord(mSmsDB, nRec);
      if(memH)
      {
        sms = (DBRecordTypePtr) MemHandleLock(memH);

        if((found = TxtFindString(sms->name, params->strToFind, &pos, &length)))
          fld = 1;
        else if((found = TxtFindString(sms->text, params->strToFind, &pos, &length)))
          fld = 2;

        if(found)
        {
          if(FindSaveMatch(params, nRec, pos, fld, length, mdbCard, mSmsDBID))
          {
            DmCloseDatabase(mSmsDB);
            return;
          }

          FindGetLineBounds(params, &rect);

          StrPrintF(year, "%d", sms->dateTime.year);
          if(StrLen(year) > 2)
          {
            year[0] = year[2];
            year[1] = year[3];
            year[2] = 0;
          }
          StrPrintF(date, "%02d.%02d.%s %02d:%02d", sms->dateTime.day, sms->dateTime.month, year, sms->dateTime.hour, sms->dateTime.minute);
          WinDrawTruncChars(sms->name, StrLen(sms->name), rect.topLeft.x+1, rect.topLeft.y, (rect.extent.x/2)-2);
          WinDrawTruncChars(date, StrLen(date), rect.topLeft.x+(rect.extent.x/2), rect.topLeft.y, (rect.extent.x/2)-2);

          params->lineNumber++;
        }
      }

      if(++nRec >= DmNumRecords(mSmsDB))
      {
        params->more = false;
        break;
      }
    }
    DmCloseDatabase(mSmsDB);
  }
}

UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags) 
{
  Err err = 0;
  UInt32 romVersion;

  FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
  if(romVersion < sysMakeROMVersion(3, 5, 0, sysROMStageRelease, 0))
  {
    FrmAlert(RomIncompatibleAlert);
    return sysErrRomIncompatible;
  }

  switch(launchCode)
  {
    case sysAppLaunchCmdNormalLaunch:
    {
      recordIds = NULL;
      if((err = StartApplication()) == 0)
      {
        EventLoop();
        StopApplication();
      }
      break;
    }

    case sysAppLaunchCmdFind:
      LookFor((FindParamsPtr) cmdPBP);
    break;

    case sysAppLaunchCmdGoTo:
      if(launchFlags & sysAppLaunchFlagNewGlobals)
      {
        recordIds = NULL;
        if((err = StartApplication()) == 0)
        {
          currentMessage = ((GoToParamsPtr) cmdPBP)->recordNum;
          FrmPopupForm(SmsForm);
          EventLoop();
          StopApplication();
        }
      }
      else
      {
        currentMessage = ((GoToParamsPtr) cmdPBP)->recordNum;
        FrmPopupForm(SmsForm);
      }
    break;

    default:
    break;
  }

  return 0;
}
